<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
    xmlns:xs="http://www.w3.org/2001/XMLSchema"
    xmlns:xd="http://www.oxygenxml.com/ns/doc/xsl"
    exclude-result-prefixes="xs xd"
    xpath-default-namespace="http://oxygenxml.com/samples/books"
    version="2.0">
    <xd:doc scope="stylesheet">
        <xd:desc>
            <xd:p><xd:b>Created on:</xd:b> May 8, 2013</xd:p>
            <xd:p><xd:b>Author:</xd:b> Octavian Nadolu</xd:p>
            <xd:p>Generates an HTML file containing a table with books.</xd:p>
        </xd:desc>
    </xd:doc>
    
    <xd:doc>
        <xd:desc>
            <xd:b>Specifies if the books should be grouped by technology.</xd:b>
        </xd:desc>
    </xd:doc>
    <xsl:param name="groupByTechnology" select="false()"/>
    
    <xd:doc>
        <xd:desc>
            <xd:p>Generates a table with books.</xd:p>
        </xd:desc>
    </xd:doc>
    <xsl:template match="books">
        <html xmlns="http://www.w3.org/1999/xhtml">
            <head>
                <title>Recommended Books</title>
            </head>
            <body>
                <h1>The list of recommended books</h1>
                <xsl:choose>
                    <xsl:when test="$groupByTechnology">
                        <!-- Generated the books grouped by categoty -->
                        <xsl:for-each-group select="book" group-by="@technology">
                            <h2>
                                <xsl:value-of select="@technology"/>
                            </h2>
                            <hr/>
                            <!-- A table is create for each group. -->
                            <table cellspacing="12px">
                                <xsl:apply-templates select="current-group()"/>
                            </table>
                        </xsl:for-each-group>
                    </xsl:when>
                    <xsl:otherwise>
                        <!-- Generates a table with books, in the order they are in the input XML document. -->
                        <table cellspacing="12px">
                            <xsl:apply-templates select="book"/>
                        </table>
                    </xsl:otherwise>
                </xsl:choose>
            </body>
        </html>
    </xsl:template>
    
    <xd:doc>
        <xd:desc>
            <xd:p>Generate a row in the table for each book.</xd:p>
        </xd:desc>
    </xd:doc>
    <xsl:template match="book">
        <!-- Book -->
        <tr>
            <td valign="top">
                <!--Image-->
                <xsl:apply-templates select="image"/>
            </td>
            <td>
                <!-- Title with link -->
                <xsl:variable name="title" select="title"/>
                <xsl:variable name="titleURI" select="@targeturi"/>
                
                <xsl:call-template name="linkTitle">
                    <xsl:with-param name="titleUri" select="$titleURI"/>
                    <xsl:with-param name="title" select="$title"/>
                </xsl:call-template>
                <!-- Author -->
                <p>
                    <i>by 
                        <xsl:value-of select="author"/>
                    </i>
                </p>
                <!-- Book info -->
                <b>Paperback: </b><xsl:value-of select="paperback"/> | 
                <b>Publisher: </b><xsl:value-of select="publisher"/> | 
                <b>Isbn: </b><xsl:value-of select="isbn"/>
                <!-- Description -->
                <xsl:apply-templates select="description"/>
            </td>
        </tr>
    </xsl:template>
    
    <xd:doc>
        <xd:desc>
            <xd:p>Generates a paragraph with the content of the description element.</xd:p>
        </xd:desc>
    </xd:doc>
    <xsl:template match="description">
        <p>
            <xsl:apply-templates/>
        </p>
    </xsl:template>
    
    <xd:doc>
        <xd:desc>
            <xd:p>Generates a title with link</xd:p>
        </xd:desc>
        <xd:param name="titleUri">The URI of the title</xd:param>
        <xd:param name="title">The value of the title</xd:param>
    </xd:doc>
    <xsl:template name="linkTitle">
        <xsl:param name="titleUri"/>
        <xsl:param name="title"/>
        <h3>
            <a href="{$titleUri}" title="{$title}">
                <xsl:value-of select="$title"/>
            </a>
        </h3>
    </xsl:template>
    
    <xd:doc>
        <xd:desc>
            <xd:p>Creates an HTML image element.</xd:p>
        </xd:desc>
    </xd:doc>
    <xsl:template match="image">
        <img src="{@src}" alt="{@description}"
            width="110" height="140" />
    </xsl:template>
</xsl:stylesheet>
